import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.print.*;
import javax.swing.text.*;
import java.awt.print.*;
import java.util.Vector;
/** Creates a Book with the Printables. This is not necessary because the whole printjob could be handled in the Printable-class.
 *  But if you use a Book (Book extends Pageable) you can choose the pages that should be printed in the print-dialog.
 * 
 * @author  Marc Seidemann 
 * @version 07.08.2007
 */
public class BookCreator {
 // --------- Felder ---------------------------------
    /** The StyledDocument that should be printed */
    private StyledDocument d;
    /** The PageFormat of the print */
    private PageFormat p;
    /** The chars of the text */
    private char[] text;
    /** The fontsize */
    private Font fontsize;
    /** The Book with all Printables */
    private Book book;
    /** The tabsize */
    private int tabsize;
    /** The numbers should be printed */
    boolean showNumbers; 
    

 // --------- Konstruktor(en) ------------------------

    /** Creates a BookCreator-object 
     *  @param d The StyledDocument that should be printed
     *  @param p The PageFormat of the print
     *  @param fontsize The fontsize of the print
     */   
        BookCreator(StyledDocument d,PageFormat p,Font fontsize,boolean showNumbers)
        {
            this.d = d;
            this.p = p;
            this.fontsize = fontsize;
            this.showNumbers = showNumbers;
            tabsize = DefReader.getTabSize();
            try
            {
                text = d.getText(0,d.getLength()).toCharArray();
            }
            catch(BadLocationException bl)
            {}
        }


 // --------- Methoden -------------------------------
    /** Creates a Book-object with all Printables */
    public void createBook()
    {
        createBook(p,text,fontsize);
    }
 
    /** Returns the Book with the Printables
     *  @return The Book
     */
    public Book getBook()
    {
        return book;
    }
    
    /** Returns a Graphics2D-object
     *  @return A Graphics2D-object
     */
    private Graphics2D get(int width,int height)
    {
        return GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().createCompatibleImage(width,height).createGraphics();
    }
    
    /** Creates the Book with the Printables and calculates all pagebreaks and linebreaks
     *  @param pf The PageFormat for the print
     *  @param text The chars of the text that should be printed
     *  @param fontsize The fontsize of the print
     */
    private void createBook(PageFormat pf,char[] text,Font fontsize)
    {
            int linesPerPage;
            int lineHeight;
            Integer[] pageBreaks;
            Integer[] lineBreaks;
            Integer[] printNumbers;
            
            Graphics2D gl = get(100,100);
            Font f = fontsize;
            gl.setFont(f);
            //The page- and linebreaks will be set
            linesPerPage = (int)((pf.getImageableHeight()-40) / gl.getFontMetrics().getHeight()) -2;
            int[] widths = gl.getFontMetrics().getWidths();
            int width = 0;
            for (int i = 0; i < widths.length; i++)
                if (widths[i] > width)
                    width = widths[i];
            int charsPerLine = (int)((pf.getImageableWidth()) / width);
            
            int pageWidth = (int)(pf.getImageableWidth());
            
            lineHeight = gl.getFontMetrics(f).getHeight();
            
            int lineCount = 0;
            int lineNumber = 0;
            int charCount = 0;
            int tabs = 0;
            boolean found = false;
            Vector linebreaks = new Vector();
            Vector pagebreaks = new Vector();
            Vector printnumbers = new Vector();
            
            printnumbers.add(new Integer(1));
            pagebreaks.add(new Integer(0));
            for(int i = 0; i < text.length; i++)
            {
                
                if (showNumbers && i == 0)
                {
                    charCount += gl.getFontMetrics().stringWidth(  String.valueOf((lineNumber+1))+getLineSpaces(lineNumber)  );
                    lineNumber++;
                }
                
                if (text[i] == '\n') //Linebreak
                {
                    
                    lineCount ++;
                    charCount = 0;
                    //The linenumbers are calculated if choosen
                    if (showNumbers)
                    {
                        charCount += gl.getFontMetrics().stringWidth(  String.valueOf((lineNumber+1))+getLineSpaces(lineNumber)  );
                        lineNumber++;
                    }
                    tabs = 0;
                    found = false;
                    if (lineCount > linesPerPage) //Pagebreak
                    {
                        pagebreaks.add(new Integer(i+1));
                        printnumbers.add(new Integer(lineNumber));
                        lineCount = 0;
                    }
                    else
                    linebreaks.add(new Integer(i+1));
                }
                else
                if (text[i] == '\r')
                {}
                else
                if ((text[i] == ' ' || text[i] == '\t') && !found)
                {
                    if (text[i] == '\t')
                        tabs += tabsize;
                    else
                        tabs++;
                    charCount += gl.getFontMetrics().charWidth(text[i]);
                }
                else
                if (text[i] == '\t') // real tab
                    charCount += tabsize * gl.getFontMetrics().charWidth(' ');
                else
                {
                    
                    found = true;
                    if (charCount+gl.getFontMetrics().charWidth(text[i]) > pageWidth) //Linebreak
                    {
                        charCount = tabs*gl.getFontMetrics().charWidth(' ');
                        found = false;
                        lineCount++;
                        
                        //The linenumbers are calculated if choosen
                        if (showNumbers)
                        {
                            charCount += gl.getFontMetrics().stringWidth(  String.valueOf((lineNumber))+getLineSpaces(lineNumber-1)  );
                        }
                        
                        if (lineCount > linesPerPage) //Pagebreak
                        {
                            pagebreaks.add(new Integer(i+1));
                            printnumbers.add(new Integer(lineNumber));
                            lineCount = 0;
                        }
                        else
                        linebreaks.add(new Integer(i));
                    }
                    charCount += gl.getFontMetrics().charWidth(text[i]);
                }
            }
            //The Lists are transformed into arrays
            pagebreaks.add(new Integer(text.length));
            pageBreaks = new Integer[pagebreaks.size()];
            pageBreaks = (Integer[])(pagebreaks.toArray(pageBreaks));
            lineBreaks = new Integer[linebreaks.size()];
            lineBreaks = (Integer[])(linebreaks.toArray(lineBreaks));
            
            printNumbers = new Integer[printnumbers.size()];
            printNumbers = (Integer[])(printnumbers.toArray(printNumbers));
            
            //The pageables will be created
            book = new Book();
            for (int i = 0; i < (pageBreaks.length-1); i++)
            {
                book.append(new DocumentPrintable(pageBreaks,lineBreaks,linesPerPage,lineHeight,d,fontsize,showNumbers,printNumbers),pf);
            }
    }
    
   
    /** Reads out the number of spaces required an returns the spaces as string */ 
    public String getLineSpaces(int number)
    {
        int digits = 4-String.valueOf((number+1)).length();
        String out = "";
        for (int i = 0; i < digits; i++)
            out += " ";
        return out;
    }
    


}// Ende der Klasse BookCreator
