import java.io.*;
import java.awt.Color;
/** Reads out the color-properties in moe.defs, the tabsize-property and the syntax-hilighting-property in bluej.defs resp. bluej.properties.
 * 
 * @author  Marc Seidemann 
 * @version 07.08.2007
 */
public class DefReader {
 // --------- Variables ---------------------------------
    /** The Path of BlueJ\Home - Directory. Is set by SyntaxSourcePrinterExtension */
    public static String DIR = "";
    /** The Path of bluej.properties . Is also set by SyntaxSourcePrinterExtension */
    public static String USERDIR = "";
 // --------- Constructor(s) ------------------------

    /** Default Konstruktor **/   
        DefReader(){}


 // --------- Methods -------------------------------
    /** Reads out the Integer of the tabsize-property
     *  @param fname The path of the property-file
     *  @param The key that should be looked up
     *  @param alt The value that should be returned if the key cannot be found
     */
    private static int readout(String fname,String key,int alt)
    {
        try
        {
            FileReader f = new FileReader(fname);
            BufferedReader reader = new BufferedReader(f);
            String last = reader.readLine();
            while(last != null)
            {
               int index = last.indexOf(key); //Search for key
               if (index > -1) //Key found
               {
                    int keyIndex = last.indexOf("=",index)+1;
                    int theKey = Integer.parseInt(last.substring(keyIndex,keyIndex+1));
                    reader.close();
                    f.close();
                    return theKey;
               }
               else
                last = reader.readLine();
            }
            reader.close();
            f.close();
        }
        catch(FileNotFoundException fn)
        {}
        catch(IOException io)
        {}
        return alt;
    }
    
    /** Reads out the color and the syntax.hilighting properties
     *  @param fname The path of the property-file
     *  @param The key that should be looked up
     *  @param alt The value that should be returned if the key cannot be found
     */
    private static String readoutColor(String fname,String key,String alt)
    {
         try
        {
            FileReader f = new FileReader(fname);
            BufferedReader reader = new BufferedReader(f);
            String last = reader.readLine();
            while(last != null)
            {
               int index = last.indexOf(key); //Search for key
               if (index > -1 && last.charAt(index+key.length()+1) == '"') //Key found
               {
                    int keyIndex = last.indexOf("=",index)+1;
                    reader.close();
                    f.close();
                    return last.substring(keyIndex);
               }
               else
                last = reader.readLine();
            }
            reader.close();
            f.close();
        }
        catch(FileNotFoundException fn)
        {}
        catch(IOException io)
        {}
        return alt;
    }
    
    /** Returns the color of the given hex-string
     *  @param hex The hex-string
     *  @return The Color that is represented by this string
     */
    private static Color createColor(String hex)
    {
        Color c = null;
        if (hex.charAt(0) == ' ')
            hex = hex.substring(1);
        int r = Integer.parseInt(hex.substring(0,2),16);
        int g = Integer.parseInt(hex.substring(2,4),16);
        int b = Integer.parseInt(hex.substring(4,6),16);

        return new Color(r,g,b);
    }

    /** Return the hex-string for a color
     *  @param key The key of the seeked color
     *  @param alt The value that should be returned if the key cannot be found
     *  @return The hex-string of the color
     */
    public static String getColorString(String key,String alt)
    {
        String out = readoutColor(DIR+"moe.defs",key,alt);
        out = out.replaceAll(" ","");
        return out;
    }
    
    /** Returns the Tabsize
     *  @param key The key for the tabsize
     *  @param alt The value that should be returned if the key cannot be found
     *  @return The tabsize 
     */
    private static int getProperty(String key,int alt)
    {
        return readout(DIR+"bluej.defs",key,alt);
    }
    
    /** Returns a BlueJ property as String
     *  @param kay The requested key
     *  @param alt The value that should be returned alternativly if the key cannot be found
     *  @return The property
     */
    public static String getPropertyString(String key,String alt)
    {
        return readoutColor(DIR+"bluej.defs",key,alt);
    }
    
    /** Returns the String of the editor.syntaxhilighting-property
     *  return The value as a string
     */
    private static String getSyntax()
    {
        return  readoutColor(USERDIR+"blueJ.properties","bluej.editor.syntaxHilighting","true");
    }
    
    /** Returns the tabsize
     *  @return The tabsize
     */
    public static int getTabSize()
    {
        return getProperty("bluej.editor.tabsize",10);
    }
    
    /** Returns if syntax-highlighting is enabled
     *  @return True if syntax-highlighting is enabled and false otherwise
     */
    public static boolean getSyntaxEnabled()
    {
        String val = getSyntax();
        val = val.toLowerCase();
        boolean output;
        if (val.equals("true"))
            output = true;
        else
            output = false;
        return output;
    }
    
    /** Returns the color for comments
     *  @return The Color for comments
     */
    public static Color getCommentColor()
    {
        return createColor(getColorString("comment","999999"));
    }

    /** Returns the color for javadoc
     *  @return The Color for javacoc
     */
    public static Color getJavadocColor()
    {
        return createColor(getColorString("javadoc","000099"));
    }
    
    /** Returns the color for stand-out
     *  @return The Color for stand-out
     */
    public static Color getBluejdocColor()
    {
        return createColor(getColorString("stand-out","ee00bb"));
    }
    
    /** Returns the color for keyword1
     *  @return The Color for keyword1
     */
    public static Color getKeyword1Color()
    {
        return createColor(getColorString("keyword1","660033"));
    }
    
    /** Returns the color for keyword2
     *  @return The Color for keyword2
     */
    public static Color getKeyword2Color()
    {
        return createColor(getColorString("keyword2","cc0000"));
    }
    
    /** Returns the color for keyword3
     *  @return The Color for keyword3
     */
    public static Color getKeyword3Color()
    {
        return createColor(getColorString("keyword3","006699"));
    }
    
    /** Returns the color for the primitive types
     *  @return The Color for the primitive types
     */
    public static Color getPrimitiveColor()
    {
        return createColor(getColorString("primitive","cc0000"));
    }
    
    /** Returns the color for strings and chars
     *  @return The Color for strings and chars
     */
    public static Color getStringColor()
    {
        return createColor(getColorString("string","006600"));
    }
    
}// End of Class DefReader
