import javax.swing.text.*;
import java.awt.print.*;
import java.awt.*;
import javax.print.*;
import java.io.*;
import bluej.extensions.*;
/** Handles the printjob.
 * 
 * @author  Marc Seidemann 
 * @version 07.08.2007
 */
public class DocumentCreator {
 // --------- Variables ---------------------------------
    /** Saves the parsed Document */
    private DefaultStyledDocument doc;
    /** The Parser that creates the StyledDocument with syntax-highlighting */
    private Parser p;
    /** Saves if the source text should be displayed with syntax-highlighing or not */
    private boolean highlighting;
    
    private static String IX = "Paper.Image.X";
    private static String IY = "Paper.Image.Y";
    private static String IWIDTH = "Paper.Image.Width";
    private static String IHEIGHT = "Paper.Image.Height";
    private static String PWIDTH = "Paper.Width";
    private static String PHEIGHT = "Paper.Height";
 // --------- Constructor(s) ------------------------

    /** Creates a DocumentCreator-object
     *  @param fname The path of the source text file
     *  @param size The fontsize of the print
     *  @param docname The name of the printjob
     *  @param showNumbers True, if the line numbers should be printed and false otherwise
     *  @param bluej Required to load the last settings to the page dialog
     */   
        DocumentCreator(String fname,Font size,String docname,boolean showNumbers,BlueJ bluej)
        {
            highlighting = DefReader.getSyntaxEnabled();   
            getDoc(fname);
            print(doc,size,docname,showNumbers,bluej);
        }


 // --------- Methods -------------------------------
  
    /** Returns a StyledDocument with the source text
     *  @param fname The path of the source text
     *  @return The StyledDocument with the source text
     */
    private StyledDocument getDoc(String fname)
    {
        load(fname);
        return doc;
    }
    
    /** Prints the Document
     *  @param d The DefaultStyledDocument that should be printed
     *  @param psize The fontsize of the print
     *  @param docname The name of the printjob
     */
    private void print(DefaultStyledDocument d,Font psize,String docname,boolean showNumbers,BlueJ bluej)
    {
        PrinterJob b = PrinterJob.getPrinterJob();
        //Tries to load the last PageFormat. If no format is found, the default is used
        PageFormat loaded = loadPageFormat(bluej);
        if (loaded == null)
            loaded = b.defaultPage();
        PageFormat pFormat = b.pageDialog(loaded);
        //Tries to save the choosen format
        savePageFormat(pFormat,bluej);
        b.setJobName(docname);
        BookCreator bookC = new BookCreator(d,pFormat,psize,showNumbers);
        bookC.createBook();
        Book book = bookC.getBook();
        b.setPageable(book);
        
        PrintService  service = PrintServiceLookup.lookupDefaultPrintService();
        try
        {
            b.setPrintService(service);
            boolean printing = b.printDialog();
            if (!printing)
                return;
            b.print();
        }
        catch(PrinterException e)
        {}
    }
    
    /** Loads the source text into the StyledDocument
     *  @param fname The path of the source text file
     */
    private void load(String fname)
    {
        try
         {
            FileInputStream f = new FileInputStream(fname);
            //the size of the array is detected
            int length = 0;
            int read = f.read();
            while(read != -1)
            {
                length++;
                read = f.read();
            }
            f.close();
            byte[] data = new byte[length];
                        
            f = new FileInputStream(fname);
            read = f.read(data);
            
            while (read != -1 && read != length)
                 read = f.read(data,read,data.length);
                 
                            
            String output = new String(data);
            f.close();
            
            if (highlighting)
            {
                p = new Parser(output);
                doc = p.parse();
            }
            else
            {
                doc = new DefaultStyledDocument();
                try
                {
                    doc.insertString(0,output,null);
                }
                catch(BadLocationException bl)
                {}
            }

        }
        catch(IOException io)
        {}
    }
    
    
    
    /** Loads the choosen PageFormat
     *  @param bluej Required to load the properties
     *  @return The PageFormat or null, if nothing is found
     */
    private PageFormat loadPageFormat(BlueJ bluej)
    {
        PageFormat pf = new PageFormat();
        Paper paper = new Paper();
        //Load value from file
        double iX = Double.parseDouble(bluej.getExtensionPropertyString(IX,"0"));
        double iY = Double.parseDouble(bluej.getExtensionPropertyString(IY,"0"));
        double iWidth = Double.parseDouble(bluej.getExtensionPropertyString(IWIDTH,"0"));
        double iHeight = Double.parseDouble(bluej.getExtensionPropertyString(IHEIGHT,"0"));
        double pWidth = Double.parseDouble(bluej.getExtensionPropertyString(PWIDTH,"0"));
        double pHeight = Double.parseDouble(bluej.getExtensionPropertyString(PHEIGHT,"0"));
        paper.setSize(pWidth,pHeight);
        paper.setImageableArea(iX,iY,iWidth,iHeight);
        pf.setPaper(paper);
        if (iX==0&&iY==0&&iWidth==0&&iHeight==0&&pWidth==0&&pHeight==0)
            return null;
        else
            return pf;
    }
    
    /** Saves the choosen PageFormat
     *  @param pf The PageFormat The properties that should be saved
     *  @param bluej Required to save the properties
     */
    private void savePageFormat(PageFormat pf,BlueJ bluej)
    {
        Paper paper = pf.getPaper();
        double iX = paper.getImageableX();
        double iY = paper.getImageableY();
        double iWidth = paper.getImageableWidth();
        double iHeight = paper.getImageableHeight();
        double pWidth = paper.getWidth();
        double pHeight = paper.getHeight();
        //Save values to file
        bluej.setExtensionPropertyString(IX,String.valueOf(iX));
        bluej.setExtensionPropertyString(IY,String.valueOf(iY));
        bluej.setExtensionPropertyString(IWIDTH,String.valueOf(iWidth));
        bluej.setExtensionPropertyString(IHEIGHT,String.valueOf(iHeight));
        bluej.setExtensionPropertyString(PWIDTH,String.valueOf(pWidth));
        bluej.setExtensionPropertyString(PHEIGHT,String.valueOf(pHeight));
    }

}// End of Class DocumentCreator