import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.print.*;
import javax.swing.text.*;
import java.awt.print.*;
import java.util.Vector;
/** The implementation of Printable. Prints the StyledDocument.
 * 
 * @author  Marc Seidemann
 * @version 07.08.2007
 */
public class DocumentPrintable implements Printable{
 // --------- Variables ---------------------------------
    /** The indices of the pagebreaks */
    private Integer[] pageBreaks;
    /** The indices of the linebreaks */
    private Integer[] lineBreaks;
    /** The chars of the text */
    private char[] text;
    /** The number of lines per page */
    private int linesPerPage;
    /** The lineheight */
    private int lineHeight;
    /** The StyledDocument that should be printed */
    private StyledDocument d;
    /** The fontsize */
    private Font fontsize;
    /** The Tabsize */
    private int tabsize;
    /** True, if the line numbers should be printed and false otherwise */
    private boolean showNumbers;
    /** The current number to print as linenumber */
    private Integer[] printNumbers;
 // --------- Constructor(s) ------------------------

    /** Creates a DocumentPrintable-object
     *  @param pageBreaks The indices of the pagebreaks
     *  @param lineBreaks The indices of the lineBreaks
     *  @param linesPerPage The number of lines per page
     *  @param lineHeight The height of a line
     *  @param d The StyledDocument that should be printed
     *  @param fontsize The fontsize of the print
     */   
        DocumentPrintable(Integer[] pageBreaks,Integer[] lineBreaks,int linesPerPage,int lineHeight,StyledDocument d,Font fontsize,boolean showNumbers,Integer[] printNumbers)
        {
            this.pageBreaks = pageBreaks;
            this.lineBreaks = lineBreaks;
            this.linesPerPage = linesPerPage;
            this.lineHeight = lineHeight;
            this.d = d;
            this.fontsize = fontsize;
            this.showNumbers = showNumbers;
            this.printNumbers = printNumbers;
            tabsize = DefReader.getTabSize();
            try
            {
                text = d.getText(0,d.getLength()).toCharArray();
            }
            catch(BadLocationException bl)
            {}
        }


 // --------- Methods -------------------------------
    /** Prints the StyledDocument
     *  @param g The Graphics-object of the PrinterJob
     *  @param pf The PageFormat of the PrinterJob
     *  @param pageIndex The page-index
     */
    public int print(Graphics g,PageFormat pf,int pageIndex)
    {
        Font f = fontsize;
        //The page does not exist
        if (pageIndex == pageBreaks.length-1)
            return NO_SUCH_PAGE;

        
        //The page exists and the printing-process starts
        int start = pageBreaks[pageIndex].intValue();        
        int end = pageBreaks[pageIndex+1].intValue();
        int printNumber = printNumbers[pageIndex].intValue();
        
        Graphics2D g2 = (Graphics2D)g;
        g2.translate(pf.getImageableX(),pf.getImageableY());
        g2.setFont(f);
        
        int y = lineHeight; //40
        int x = 0;
        int tabs = 0;
        int linenumber = pageIndex * linesPerPage;
        boolean found = false;
        for (int i = start; i < end; i++)
        {
            //The line numbers are printed for the first line on page, if the option is choosen
            if (showNumbers && i == start)
            {
                Color farbe = g2.getColor();
                g2.setColor(Color.black);
                g2.drawString( String.valueOf(printNumber),x,y);
                printNumber++;
                g2.setColor(farbe);
                x = g.getFontMetrics().stringWidth(  String.valueOf(printNumber)+getLineSpaces(printNumber)  );
            }
            
            
            //The Linebreak is calculated
            if ( linenumber < lineBreaks.length && i == lineBreaks[linenumber].intValue() )
            {
                y += lineHeight;
                //The line numbers are printed, if the option is choosen
                if (showNumbers)
                {
                    if (i > 0 && text[i-1] == '\n')
                    {
                        x = 0;
                        Color farbe = g2.getColor();
                        g2.setColor(Color.black);
                        g2.drawString( String.valueOf(printNumber),x,y);
                        g2.setColor(farbe);
                        x += g.getFontMetrics().stringWidth(  String.valueOf(printNumber)+getLineSpaces(printNumber) )+ tabs * g.getFontMetrics().charWidth(' ');
                        printNumber++;
                    }
                    else
                        x = g.getFontMetrics().stringWidth(  String.valueOf(printNumber)+getLineSpaces(printNumber)  ) + tabs * g.getFontMetrics().charWidth(' ');
                }
                else
                    x = tabs * g.getFontMetrics().charWidth(' ');
                linenumber++;
                found = false;
            }
            
            if (text[i] == '\t' && found) //real tab
            {
                x += tabsize*g.getFontMetrics().charWidth(' ');
                continue;
            }
            
            if ((text[i] == ' ' || text[i] == '\t') && !found)
            {
                if (text[i] == '\t')
                {
                   tabs += tabsize;
                   continue;
                }
                else
                    tabs++;
            }
            else
                found = true;
            
            if (text[i] == '\n' || text[i] == '\r')
            {
                tabs = 0;
            }

            else
            {
                Color farbe = StyleConstants.getForeground(d.getCharacterElement(i).getAttributes());
                g2.setColor(farbe);
                g2.drawString(String.valueOf(text[i]),x,y);
                x += g.getFontMetrics().charWidth(text[i]);
            }
              
        }
        y = (int)(pf.getImageableHeight());
        g2.setColor(Color.black);
        x = (int)(pf.getImageableWidth()/2) - g.getFontMetrics().charWidth('/') - 2*g.getFontMetrics().charWidth('0');
        g2.drawString((pageIndex+1)+"/"+(pageBreaks.length-1),x,y);
        return PAGE_EXISTS;
    }
    

    /** Reads out the number of spaces required an returns the spaces as string */ 
    public String getLineSpaces(int number)
    {
        int digits = 4-String.valueOf(number).length();
        String out = "";
        for (int i = 0; i < digits; i++)
            out += " ";
        return out;
    }

}// End of Class DocumentPrintable
