import java.io.*;
import java.awt.*;
import javax.swing.text.*;
import javax.swing.text.rtf.*;
import javax.swing.*;
/** Creates the HTML and RTF files.
 * 
 * @author  Marc Seidemann
 * @version 07.09.2007
 */
public class Export {
 // --------- Fields ---------------------------------
    /** The saves line numbers that are printed */
    static int lineCounter;
    /** The colors for the keywords */
    private static Color commentColor;
    private static Color javadocColor;
    private static Color bluejdocColor;
    private static Color stringColor;
    private static Color keyword1Color;
    private static Color keyword2Color;
    private static Color keyword3Color;
    private static Color primitiveColor;
 // --------- Constructor(s) ------------------------

    /** Default Constructor **/   
        Export(){}

 // --------- Methods -------------------------------
  
 
    /** Exports the java File to a HTML file
     *  @param s The path of the java file that should be exportet.
     *  @param f The font for the HTML file.
     *  @param name The title of the HTML file.
     */
    public static void initHTMLOld(String s,Font f,String name,boolean linenumber)
    {
        JFileChooser chooser = new JFileChooser();
        boolean createNew = false;
        boolean reference = false;
        String newCSSPath="";
        String referencePath="";
        String htmlPath="";
        
        //New CSS file (???)
        int val = JOptionPane.showConfirmDialog(null,"Create new CSS file for styles?");
        if (val == JOptionPane.OK_OPTION)
        {
            chooser.setSelectedFile(new File("javacode.css"));
            val = chooser.showSaveDialog(null) ;
            if (val == JFileChooser.APPROVE_OPTION)
            {
                if (chooser.getSelectedFile().exists())
                {
                    //The specified CSS file already exists
                    val = JOptionPane.showConfirmDialog(null,"Overwrite existing CSS?");
                    if (val == JOptionPane.OK_OPTION)
                        createNew = true;
                    else
                    if (val == JOptionPane.CANCEL_OPTION)
                        return;
                }
                else
                    createNew = true;
                newCSSPath = chooser.getSelectedFile().getAbsolutePath();
            }
                
        }
        else
        if (val == JOptionPane.CANCEL_OPTION)
            return;
        
        //Reference CSS file for XHTML (???)
        val = JOptionPane.showConfirmDialog(null,"Reference external CSS file in the XHTML file?");
        if (val == JOptionPane.OK_OPTION)
        {
            chooser.setSelectedFile(new File("javacode.css"));
            val = chooser.showOpenDialog(null);
            if (val == JFileChooser.APPROVE_OPTION)
            {
                reference = true;
                referencePath = chooser.getSelectedFile().getAbsolutePath();
            }
        }
        else
        if (val == JOptionPane.CANCEL_OPTION)
            return;
        
        //Save the XHTML
        chooser.setSelectedFile(new File(name+".java.htm"));
        val = chooser.showSaveDialog(null);
        if (val == JFileChooser.APPROVE_OPTION)
        {
            htmlPath = chooser.getSelectedFile().getAbsolutePath();
        }
        else
        if (val == JFileChooser.CANCEL_OPTION)
            return;
            
        String source = loadSourceFromFile(s);
        
        //Creates the XHTML
       // createHTML(new Parser(source).parse(),f,name,linenumber,htmlPath,createNew,reference,newCSSPath,referencePath);
    }
    
    
    /** Exports the java File to a HTML file
     *  @param s The path of the java file that should be exportet.
     *  @param f The font for the HTML file.
     *  @param name The title of the HTML file.
     */
    public static void initHTML(String s,Font f,String name,boolean linenumber)
    {
        JFileChooser chooser = new JFileChooser();
        boolean createNew = true;
        chooser.setSelectedFile(new File(name+".java.htm"));
        int val = chooser.showSaveDialog(null) ;
        if (val == JFileChooser.APPROVE_OPTION)
        {
            String htmlPath = chooser.getSelectedFile().getAbsolutePath();
            htmlPath = htmlPath.replace('\\','/');
            String css = htmlPath.substring(0,htmlPath.lastIndexOf("/")+1)+"ssp-javacode.css";
            File cssf = new File(css);
            if (cssf.exists())
                createNew = false;
            String cssPath = cssf.getAbsolutePath();
            String source = loadSourceFromFile(s);
            createHTML(new Parser(source).parse(),f,name,linenumber,htmlPath,createNew,css);
        }
    }
    
    /** Creates the HTML code
     *  @param d The DefaultStyledDocument with the parsed source text.
     *  @param f The font for the HTML file.
     *  @param name The title for the HTML file.
     *  @param css The name for the css file.
     *  @param writecss True, if the css file should be created and false otherwise.
     *  @param path The path to which the HTML file should be written to.
     */
    //private static void createHTML(StyledDocument d,Font f,String name,boolean linenumber,String path,boolean writeCSS,boolean referenceCSS,String writePath,String referencePath)
    private static void createHTML(StyledDocument d,Font f,String name,boolean linenumber,String path,boolean writeCSS,String writePath)
    {
        initColors();
        Color color=null;
        String text="<?xml version=\"1.0\" ?>\r\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\r\n\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\r\n<html xmlns=\"http://www.w3.org/1999/xhtml\">\r\n<head>\r\n<title>"+name+"</title>\r\n<style type=\"text/css\">\r\n"+"@import \"ssp-javacode.css\";\r\n"+createStyles(f)+"</style>\r\n\r\n";
        
        //if (referenceCSS)
        //text+="<link rel=\"stylesheet\" type=\"text/css\" href=\"file:///"+referencePath.replace('\\','/')+"\">\r\n";
            
        text += "</head>\r\n<body>\r\n<pre>\r\n" ;
        
        if (linenumber)
            text+="<span class=\"lnNo\">1</span>   ";
        char[] chars;
        try
        {
            FileOutputStream outstream = new FileOutputStream(path); //writes the file
            BufferedOutputStream out = new BufferedOutputStream(outstream);
            byte[] data;
            
            data = text.getBytes();
            out.write(data);
            text="";
            
            chars=d.getText(0,d.getLength()).toCharArray();
            lineCounter = 2;
            for (int i = 0;i < chars.length; i++)
            {
                if (color != null && StyleConstants.getForeground(d.getCharacterElement(i).getAttributes()).equals(color))
                {
                    text=convert(chars[i],getColor(color),linenumber);
                    data = text.getBytes();
                    out.write(data);
                    text="";
                }
                else
                {
                    if (color != null && !getColor(color).equals("") )
                        text+="</span>";
                    color = StyleConstants.getForeground(d.getCharacterElement(i).getAttributes());
                    String currColor = getColor(color);
                    if (!currColor.equals(""))
                        text+="<span class=\""+currColor+"\">";
                    text+=convert(chars[i],getColor(color),linenumber);
                    data = text.getBytes();
                    out.write(data);
                    text = "";
                }
            }
            text+="\n</span>\n</pre>\n</body>\n</html>";
            data = text.getBytes();
            out.write(data);
            out.close();
            
            // Creates a CSS file
            if(writeCSS)
            {
                outstream = new FileOutputStream(writePath);
                out = new BufferedOutputStream(outstream);
                data = createStyles(f).getBytes();
                out.write(data);
                out.close();
            }
        }
        catch(Exception e)
        {}
        
    }
    
    /** Returns the id for the tags to display the color in HTML files.
     *  @param The color for the requested keyword. 
     */
    private static String getColor(Color c)
    {
        String output="";
        if (c.equals(commentColor))
            output="comment";
        else
        if (c.equals(javadocColor))
            output="javadoc";
        else
        if (c.equals(bluejdocColor))
            output="bluejdoc";
        else
        if (c.equals(keyword1Color))
            output="keyword1";
        else
        if (c.equals(keyword2Color))
            output="keyword2";
        else
        if (c.equals(keyword3Color))
            output="keyword3";
        else
        if (c.equals(primitiveColor))
            output="primitive";
        else
        if (c.equals(stringColor))
            output="string";
        return output;
    }
    
    /** Creates the part with the css
     *  @param f The font for the HTML file.
     *  @return The css part of the HTML file.
     */
    private static String createStyles(Font f)
    {
        String out="";
        out+="pre {\nfont-size:"+String.valueOf(f.getSize())+";\r\nfont-family:"+f.getName()+";\ncolor:black;\r\n";
        if (f.isItalic())
            out += "font-style:italic;\r\n";
        if (f.isBold())
            out += "font-weight:bold;\r\n";
        out += "}\r\n\r\n";
        out+="span.keyword1 {\r\ncolor:#"+DefReader.getColorString("keyword1","660033")+";\r\n}\r\n\r\n";
        out+="span.keyword2 {\r\ncolor:#"+DefReader.getColorString("keyword2","cc0000")+";\r\n}\r\n\r\n";
        out+="span.keyword3 {\r\ncolor:#"+DefReader.getColorString("keyword3","006699")+";\r\n}\r\n\r\n";
        out+="span.primitive {\r\ncolor:#"+DefReader.getColorString("primitive","cc0000")+";\r\n}\r\n\r\n";
        out+="span.comment {\r\ncolor:#"+DefReader.getColorString("comment","999999")+";\r\n}\r\n\r\n";
        out+="span.javadoc {\r\ncolor:#"+DefReader.getColorString("javadoc","000099")+";\r\n}\r\n\r\n";
        out+="span.bluejdoc {\r\ncolor:#"+DefReader.getColorString("stand-out","ee00bb")+";\r\n}\r\n\r\n";
        out+="span.string {\r\ncolor:#"+DefReader.getColorString("string","006600")+";\r\n}\r\n\r\n";
        out+="span.lnNo {\r\n\r\n}\r\n\r\n";
        
        return out;
    }
    
    /** Converts a char in a HTML conform String
     *  @param c The char that should be translated.
     *  @return The HTML conform String (maybe the char itself as String, if it was conform) 
     */
    private static String convert(char c,String color,boolean linenumber)
    {
        String out=String.valueOf(c);
        switch(c)
        {
            case '': out="&auml;";break;
            case '': out="&ouml;";break;
            case '': out="&uuml;";break;
            case '': out="&Auml;";break;
            case '': out="&Ouml;";break;
            case '': out="&Uuml;";break;
            
            case '': out="&szlig";break;
            case '\"': out="&quot;";break;
            case '&': out="&amp;";break;
            case '>': out="&gt;";break;
            case '<': out="&lt;";break;
            case ' ': out=" ";break;
            case '\t': out = getTabs();break;
            case '\n': if (linenumber)
                       {
                           out = "";
                           if (!color.equals(""))
                            out+="</span>";
                           out += "\r\n";
                           out+="<span class=\"lnNo\">";
                           out+=String.valueOf(lineCounter);
                           out+="</span>";
                           if (lineCounter >= 10) 
                            out+="  ";
                           else 
                            out+= "   ";
                           if (!color.equals(""))
                            out+="<span class=\""+color+"\">";
                           lineCounter++;
                       }
                       else
                        out = "\r\n";
                       break;
                       
            case '': out = "&copy;";break;
            case '\r': out = "";
        }
        return out;
    }
    
    /** Returns the tabs as String.
     *  @return The tabs as HTML String.
     */
    private static String getTabs()
    {
        String out="";
        int size = DefReader.getTabSize();
        for(int i = 0; i < size; i++)
            out+="&nbsp;";
        return out;
    }
    
    
    /** Loads the HTML form a java file.
     *  @param fname The path of the java file
     */
    private static String loadSourceFromFile(String fname)
    {
        String output ="";
        try
         {
            FileInputStream f = new FileInputStream(fname);
            //the size of the array is detected
            int length = 0;
            int read = f.read();
            while(read != -1)
            {
                length++;
                read = f.read();
            }
            f.close();
            byte[] data = new byte[length];
                        
            f = new FileInputStream(fname);
            read = f.read(data);
            
            while (read != -1 && read != length)
                 read = f.read(data,read,data.length);
                 
                            
            output = new String(data);
            f.close();
        }
        catch(Exception e)
        {}
        return output;
    }
    
    /** Exports to RTF
     *  @param path The path of the java file that should be exported
     *  @param name The title of the file
     */
    public static void initRTF(String path,Font f,String name)
    {
        JFileChooser chooser = new JFileChooser();
        chooser.setSelectedFile(new File(name+".java.rtf"));
        int val = chooser.showSaveDialog(null);
        if (val == chooser.APPROVE_OPTION)
        {
            File file = chooser.getSelectedFile();
            String target = file.getAbsolutePath();
            createRTF(path,target,f);
        }
    }

    
    /** Creates an RTF file
     *  @param sourcename The name of the .java source file
     *  @param targetname The name of the .rtf target file
     */
    private static void createRTF(String sourcename,String targetname,Font f)
    {
        try
        {
            Parser p = new Parser( loadSourceFromFile(sourcename), f );
            StyledDocument d = p.parse();
            String text = d.getText(0,d.getLength());
            RTFEditorKit k = new RTFEditorKit();
            FileOutputStream out = new FileOutputStream(targetname);
            k.write(out,d,0,d.getLength());
            out.close();
        }
        catch(Exception e)
        {}
    }

    
    /** Reads out the colors from the defs */
    private static void initColors()
    {
        commentColor = DefReader.getCommentColor();
        javadocColor = DefReader.getJavadocColor();
        bluejdocColor = DefReader.getBluejdocColor();
        keyword1Color = DefReader.getKeyword1Color();
        keyword2Color = DefReader.getKeyword2Color();
        keyword3Color = DefReader.getKeyword3Color();
        primitiveColor = DefReader.getPrimitiveColor();
        stringColor = DefReader.getStringColor();
    }
    
}// End of Class HTMLCreator