import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
/** The panel shown in the BlueJ preference panel.
 * 
 * @author  Marc Seidemann 
 * @version 07.09.2007
 */
public class FontDialog extends JPanel{
 // --------- Fields ---------------------------------
    /** Just some panels.They are needed to structure the BorderLayout. */
    JPanel fontPanel;
    JPanel editPanel;
    JPanel selectablePanel;
    JPanel fontEditPanel;
    JPanel checkPanel;
    JPanel buttonPanel;
    JPanel previewPanel;
    JPanel preview;
    JPanel refreshPanel;
    
    /** The ComboBox with the font names */
    JComboBox fontChooser;
    /** The Label for the ComboBox */
    JLabel fontChooserLabel;
    /** The TextField with the font size */
    JTextField size;
    /** The Label for the TextField with the font size */
    JLabel sizeLabel;
    /** The CheckBox for the italic style option */
    JCheckBox italic;
    /** The CheckBox for the bold style option */
    JCheckBox bold;
    /** The CheckBox for the line numbers */
    JCheckBox number;
    /** The Button that restores the settings */
    JButton defaultSettings;
    /** The Button that refreshes the view of the font */
    JButton refresh;
    /** The Label for the Panel with the preview */
    JLabel previewPanelLabel;
    /** The Label that previews the font */
    JLabel previewLabel;
    /** Listens for the ComboBox and the CheckBoxes */
    ItemChangeListener il;
 // --------- Constructor(s) ------------------------

    /** Default Constructor **/   
        FontDialog()
        {
            super();
            setLayout(new BorderLayout());
            setSize(327,183);
            init();
            setVisible(true);
        }


 // --------- Methods -------------------------------
    /** Returns the font for printing
     *  @return The font for printing.
     */
    public Font getPrintFont()
    {
        previewLabel.setFont( new Font(fontChooser.getSelectedItem().toString(),getFontStyle(),getFontSize()) );
        return previewLabel.getFont();
    }
    
    /** Sets the fontname for printing
     *  @param fontname The fontname for printing.
     */
    public void setFontname(String fontname)
    {
        fontChooser.getModel().setSelectedItem(fontname);
        previewLabel.setFont( new Font(fontChooser.getSelectedItem().toString(),getFontStyle(),getFontSize()) );
    }
    
    /** Sets the fontsize for printing
     *  @param The fontsize for printing.
     */
    public void setFontsize(String fontsize)
    {
        size.setText(fontsize);
        previewLabel.setFont( new Font(fontChooser.getSelectedItem().toString(),getFontStyle(),getFontSize()) );
    }
    
    /** Sets the style for printing
     *  @param The style for printing. 
     */
    public void setStyle(String style)
    {
        int occur;
        occur = style.indexOf("b");
        if (occur > -1)
            bold.setSelected(true);
            
        occur = style.indexOf("i");
        if (occur > -1)
            italic.setSelected(true);
            
        occur = style.indexOf("n");
        if (occur > -1)
            number.setSelected(true);       
    }
    
    
    /** Returns if line numbers should be printed */
    public boolean getLinenumberPrinting()
    {
        return number.isSelected();
    }
    
    /** Returns the fontname for printing
     *  @return The fontname for printing.
     */
    public String getFontname()
    {
        return fontChooser.getModel().getSelectedItem().toString();
    }
    
    /** Returns the fontsize for printing
     *  @return the fontsize for printing as a String.
     */
    public String getFontsize()
    {
        return size.getText();
    }
    
    /** Returns the style for printing
     *  @return the style for printing (bold,italic).
     */
    public String getStyle()
    {
        String out = "";
        if (bold.isSelected())
            out += "b";
        if (italic.isSelected())
            out += "i";
        if (number.isSelected())
            out += "n";
            return out;
    }
 
    /** Initialises all components */
    private void init()
    {
        il = new ItemChangeListener();
        String fonts[] = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
        
        fontPanel = new JPanel();
        fontPanel.setLayout(new BorderLayout());
        fontPanel.setSize(300,20);
        fontPanel.setVisible(true);
        add(fontPanel,BorderLayout.NORTH);
        
        fontChooserLabel = new JLabel("The font for printing:");
        fontChooserLabel.setVisible(true);
        fontPanel.add(fontChooserLabel,BorderLayout.NORTH);
        
        fontChooser = new JComboBox();
        fontChooser.setModel(new DefaultComboBoxModel(fonts));
        fontChooser.setSize(300,20);
        fontChooser.addItemListener(il);
        fontChooser.setVisible(true);
        fontPanel.add(fontChooser,BorderLayout.CENTER);
        
        editPanel = new JPanel();
        editPanel.setLayout(new BorderLayout());
        editPanel.setSize(300,20);
        editPanel.setVisible(true);
        add(editPanel,BorderLayout.SOUTH);
        
        
        selectablePanel = new JPanel();
        selectablePanel.setLayout(new BorderLayout());
        selectablePanel.setVisible(true);
        editPanel.add(selectablePanel,BorderLayout.NORTH);
        
        checkPanel = new JPanel();
        checkPanel.setLayout(new BorderLayout());
        checkPanel.setVisible(true);
        selectablePanel.add(checkPanel,BorderLayout.WEST);
        
        italic = new JCheckBox("italic");
        italic.setSize(80,20);
        italic.addItemListener(il);
        italic.setVisible(true);
        checkPanel.add(italic,BorderLayout.WEST);
        
        bold = new JCheckBox("bold");
        bold.setSize(80,20);
        bold.addItemListener(il);
        bold.setVisible(true);
        checkPanel.add(bold,BorderLayout.CENTER);
        
        number = new JCheckBox("Print line numbers");
        number.setVisible(true);
        checkPanel.add(number,BorderLayout.EAST);

        fontEditPanel = new JPanel();
        fontEditPanel.setLayout(new BorderLayout());
        fontEditPanel.setVisible(true);
        selectablePanel.add(fontEditPanel,BorderLayout.EAST);

        sizeLabel = new JLabel("The font size for printing:");
        sizeLabel.setSize(100,20);
        sizeLabel.setVisible(true);
        fontEditPanel.add(sizeLabel,BorderLayout.NORTH);
        
        size = new JTextField("12");
        size.setSize(80,20);
        size.setVisible(true);
        fontEditPanel.add(size,BorderLayout.SOUTH);
        
        refreshPanel = new JPanel();
        refreshPanel.setLayout(new BorderLayout());
        refreshPanel.setVisible(true);
        selectablePanel.add(refreshPanel,BorderLayout.CENTER);
        
        
        refresh = new JButton("Refresh View");
        refresh.setSize(100,20);
        refresh.addActionListener(il);
        refresh.setVisible(true);
        refreshPanel.add(refresh,BorderLayout.SOUTH);
        
        previewPanel = new JPanel();
        previewPanel.setLayout(new BorderLayout());
        previewPanel.setVisible(true);
        add(previewPanel,BorderLayout.CENTER);
        
        previewPanelLabel = new JLabel("Preview");
        previewPanelLabel.setVisible(true);
        previewPanel.add(previewPanelLabel,BorderLayout.NORTH);
        
        preview = new JPanel();
        preview.setLayout(new BorderLayout());
        preview.setBackground(Color.white);
        preview.setSize(300,100);
        preview.setVisible(true);
        previewPanel.add(preview,BorderLayout.CENTER);
        
        
        previewLabel = new JLabel("This is an example text.");
        previewLabel.setSize(300,100);
        previewLabel.setVisible(true);       
        preview.add(previewLabel,BorderLayout.CENTER);
        
        
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BorderLayout());
        buttonPanel.setVisible(true);
        editPanel.add(buttonPanel,BorderLayout.SOUTH);
        
        defaultSettings = new JButton("Set to BlueJ default settings");
        defaultSettings.setSize(150,20);
        defaultSettings.addActionListener(il);
        defaultSettings.setVisible(true);
        buttonPanel.add(defaultSettings,BorderLayout.CENTER);
        
        previewLabel.setFont( new Font(fontChooser.getSelectedItem().toString(),getFontStyle(),getFontSize()) );
    }
    
    /** Returns the fontsize.
     *  @return the fontsize as an integer.
     */
    private int getFontSize()
    {
        int output = 12;
        try
        {
            output = Integer.parseInt(size.getText());
        }
        catch(Exception e)
        {
            output = 12;
            size.setText(String.valueOf(output));
        }
        return output;
    }
    
    /** Returns the fontstyle.
     *  @return the fontstyle as an integer.
     */
    private int getFontStyle()
    {
        int output = Font.PLAIN;
        if (bold.isSelected() && italic.isSelected())
            output = (Font.BOLD|Font.ITALIC);
        else
        if (bold.isSelected())
            output = Font.BOLD;
        else
        if (italic.isSelected())
            output = Font.ITALIC;
        return output;
    }
    
    
    //--------------- Private Classes ---------------------------
    
    
    /** The ItemListener. */
    private class ItemChangeListener implements ItemListener,ActionListener
    {
        public void itemStateChanged(ItemEvent e)
        {
            if (e.getSource() == fontChooser)
                previewLabel.setFont( new Font(e.getItem().toString(),getFontStyle(),getFontSize()) );
            else
                previewLabel.setFont( new Font(fontChooser.getSelectedItem().toString(),getFontStyle(),getFontSize()) );
            repaint();
        }

        public void actionPerformed(ActionEvent e)
        {
            if (e.getSource() == refresh)
            {
                previewLabel.setFont( new Font(fontChooser.getSelectedItem().toString(),getFontStyle(),getFontSize()) );
                repaint();
            }
            else
            if (e.getSource() == defaultSettings)
            {
                String fontname = DefReader.getPropertyString("bluej.editor.font","SansSerif");
                int boldInt = fontname.indexOf("-bold");
                int style = Font.PLAIN;
                italic.setSelected(false);
                if (boldInt > -1)
                {
                    style = Font.BOLD;
                    bold.setSelected(true);
                    fontname = fontname.substring(0,boldInt);
                }
                else
                    bold.setSelected(false);
                
                String fontsize = DefReader.getPropertyString("bluej.editor.fontsize","12");
                size.setText(fontsize);
                int fsize = Integer.parseInt(fontsize);
                fontChooser.getModel().setSelectedItem(fontname);
                previewLabel.setFont(new Font(fontname,style,fsize));
            }
        }
    }


}// End of Class FontDialog
