import java.awt.*;
import java.util.Vector;
import javax.swing.text.*;
/** Parser for Java-Syntax.
 * 
 * @author  Marc Seidemann
 * @version 02.08.2007
 */
public class Parser {
 // --------- Variables ---------------------------------
    /** The Keywords */
    private Vector keys;
    /** The cars of the source text */
    private char[] chars;
    /** The chars that are ignored in the context of a keyword */
    private char[] ignore = {';',':',',','(','[','{',')',']','}','=','&','&','\r','\t','|','|','%','+','-','.','!','<','>','?',' ','\n'};

    /** The Color for javadoc */
    private Color docColor = DefReader.getJavadocColor();
    /** The Color for comments */
    private Color commentColor = DefReader.getCommentColor();
    /** The Color for strings and chars */
    private Color stringColor = DefReader.getStringColor();
    /** The Color for stand-out (BlueJ comments) */
    private Color bluejColor = DefReader.getBluejdocColor();

     /** The Keywords of type keyword1 */
     private String[] darkred = {"abstract","do","public","protected","private","synchronized","try","catch","for","while","switch","case","break","return","continue","instanceof","new","final","static","if","else","default"};
     /** The Keywords of type keyword2 */
     private String[] red = {"package","import","implements","extends","class","interface"};
     /** The Keywords of type primitive */
     private String[] prim = {"void","int","char","long","double","boolean","float","byte"};
     /** The Keywords of type keyword3 */
     private String[] blue = {"true","false","super","this","null"};
     
     /** The StyledDocument that should be the result of the parsing-process */
     private DefaultStyledDocument d;
     
     private Font font;
 // --------- Constructor(s) ------------------------

    /** Creates a Parser-object
     *  @param chars The source text
     */   
        Parser(String chars)
        {
            this.chars = chars.toCharArray();
            d = new DefaultStyledDocument();
            keys = new Vector();
            keys.add(new KeyWords(darkred,DefReader.getKeyword1Color()));
            keys.add(new KeyWords(red,DefReader.getKeyword2Color()));
            keys.add(new KeyWords(blue,DefReader.getKeyword3Color()));
            keys.add(new KeyWords(prim,DefReader.getPrimitiveColor() ) );
            font = new Font("Arial",Font.PLAIN,12);
        }
        
        Parser(String chars,Font f)
        {
            this(chars);
            font = f;
        }

 // --------- Methods -------------------------------
    /** Parses the source text and returns a DefaultStyledDocument with the highlighted source text */
    public DefaultStyledDocument parse()
    {
        
        SimpleAttributeSet s = new SimpleAttributeSet();
        StyleConstants.setBold(s,font.isBold());
        StyleConstants.setItalic(s,font.isItalic());
        StyleConstants.setFontFamily(s,font.getName());
        StyleConstants.setFontSize(s,font.getSize());
        
        boolean stringFound = false;
        boolean commentFound = false;
        boolean charFound = false;
        boolean docFound = false;
        boolean oneLineComment = false;
        Color c = Color.black;

        String lastKeyword = "";
        for(int i = 0; i < chars.length; i++)
        {
            
            boolean keyFound = false;
            
            if (chars[i] == '\"' && !((i > 0 && chars[i-1] == '\\') && !(i > 1 && chars[i-2] == '\\')) && !charFound && !docFound && !commentFound) //A string has been found
            {
                if (stringFound) //String ends
                {
                    stringFound = false;
                    StyleConstants.setForeground(s,c);
                    
                    try
                    {
                        d.insertString(d.getLength(),String.valueOf(chars[i]),s);
                    }
                    catch(Exception e)
                    {}
                    continue;
                    
                }
                else //New string found
                {
                    c = stringColor;
                    stringFound = true;
                }
                
                
            }

            if (chars[i] == '\'' && !((i > 0 && chars[i-1] == '\\') && !(i > 1 && chars[i-2] == '\\')) && !stringFound && !docFound && !commentFound) //A char has been found
            {
                if (charFound) //Char ends
                {
                    charFound = false;
                    StyleConstants.setForeground(s,c);
                   
                    try
                    {
                        d.insertString(d.getLength(),String.valueOf(chars[i]),s);
                    }
                    catch(Exception e)
                    {}
                    continue;
                } 
                else //New char
                {
                    c = stringColor;
                    charFound = true;
                }
                
                
            }
            
            if (i < chars.length-2 && chars[i] == '/'  && chars[i+1] == '*' && chars[i+2] == '*' && !stringFound && !commentFound && !docFound) //A javadoc has been found
            {
                c = docColor;
                docFound = true;
            }
            
            if (i < chars.length-2 && chars[i] == '/'  && chars[i+1] == '*' && chars[i+2] == '#' && !stringFound && !commentFound && !docFound ) //A bluejdoc has been found
            {
                c = bluejColor;
                docFound = true;
            }
            
            if (i < chars.length-1 && chars[i] == '/' && (chars[i+1] == '*' || chars[i+1] == '/') && !stringFound && !docFound) //A comment has been found
            {
                if (chars[i+1] == '/')
                    oneLineComment = true;
                c = commentColor;
                commentFound = true;
                
            }
            
            if (( (chars[i] == '/' && i > 0 && chars[i-1] == '*') || (chars[i] == '\n' && oneLineComment) && !stringFound && (commentFound||docFound)))//A comment or a javadoc ends
            {
                if (oneLineComment && chars[i] != '\n')
                {}
                else
                {
                    StyleConstants.setForeground(s,c);
                    
                    commentFound = false;
                    if (chars[i] == '\n')
                        oneLineComment = false;
                    docFound = false;
                    try
                    {
                        d.insertString(d.getLength(),String.valueOf(chars[i]),s);
                    }
                    catch(Exception e)
                    {}
                    continue;
                }
            }
            
            //No tag found, begins searching for keywords.
            if (!stringFound && !commentFound && !charFound && !docFound) 
            {
                c = Color.black;
                if (isIgnoreCase(chars[i])) //A character that should be ignored in context of a keyword
                {
                    for (int k = 0; k < keys.size(); k++)
                    if (((KeyWords)(keys.elementAt(k))).hasKeyword(lastKeyword)) //the last word is a keyword
                    {
                        c = ((KeyWords)(keys.elementAt(k))).getColor();
                        break;
                    }
                    keyFound = true;
                }
                else
                    lastKeyword += chars[i]; //The last char is not ignored
            }
                        


            StyleConstants.setForeground(s,c);
            try
            {
                
                if (keyFound || i == chars.length-1)
                {
                    d.insertString(d.getLength(),lastKeyword,s);
                    if (i == chars.length-1)
                    {
                        StyleConstants.setForeground(s,c);
                        d.insertString(d.getLength(),String.valueOf(chars[i]),s);
                        break;
                    }
                        
                 
                    StyleConstants.setForeground(s,Color.black);
                    d.insertString(d.getLength(),String.valueOf(chars[i]),s);
                    lastKeyword = "";
                }
                if (stringFound || commentFound || charFound || docFound)
                {
                    StyleConstants.setForeground(s,c);
                    
                    d.insertString(d.getLength(),String.valueOf(chars[i]),s);
                    continue;
                }
                
            }
            catch(Exception e)
            {}
        }
        return d;
    }

    /** Returns if a char is ignored in the context of a keyword
     *  @param c The char that should be tested
     */
    private boolean isIgnoreCase(char c)
    {
        for (int i = 0; i < ignore.length; i++)
            if (c == ignore[i])
                return true;
        return false;
    }
     
   
    /* ----------------------------- Private Classes -------------------------------- */
    
    
    /**
     * The Keywords.
     */
    private class KeyWords
    {
    
        // ------------------- Variables -------------------------
        /** The Keywords */
        String[] keys;
        /** The Color of the Keywords */
        Color color;

        // --------------- Constructor ----------------------


        /**
         * Creates a KeyWords-Objekt with the Keywords and the highlighting-color
         * @param key The array with the keywords
         * @param color The color of the keywords
         */
        public KeyWords(String[] keys,Color color)
        {
            this.keys = keys;
            this.color = color;
        }


        //------------------- Methods -------------------------
        /** Returns if the given string is a keyword
         *  @param s The string that should be testes
         *  @return True if this string is a keyword and false otherwise
         */
        public boolean hasKeyword(String s)
        {
            for (int i = 0; i < keys.length; i++)
                if (keys[i].equals(s))
                    return true;
            return false;
        }
    
        /** Returns the Color of this keyword-type
         *  @return The Color
         */
        public Color getColor()
        {
            return color;
        }
    }
    //End of Class KeyWords
    
    
    

}//End of Class Parser
