import bluej.extensions.*;
import bluej.extensions.event.*;
import bluej.extensions.editor.*;

import java.net.URL;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/**
 * BlueJ-Extension SyntaxSourcePrinterExtension.
 */

public class SyntaxSourcePrinterExtension extends Extension implements PackageListener {

    /**
     * Starts the Extension
     */
    public void startup (BlueJ bluej)
    {
        /** Das Men wird erzeugt */
        // The BlueJ-Paths are set
        DefReader.DIR = bluej.getSystemLibDir().getAbsolutePath()+System.getProperty("file.separator");
        DefReader.USERDIR = bluej.getUserConfigDir().getAbsolutePath()+System.getProperty("file.separator");
        
        bluej.setMenuGenerator(new MenuBuilder(bluej));

        /** Die Eintrge im Einstellungs-Fenster werden erzeugt */
        Preferences myPreferences = new Preferences(bluej);
        bluej.setPreferenceGenerator(myPreferences);
        /** Der Package-Listener fr diese Extension wird hinzugefgt */
        bluej.addPackageListener(this);
    }

    /**
     * A Package has been opened
     */
    public void packageOpened ( PackageEvent ev ) 
    {}  
  
    /**
     * A Package has been closed
     */
    public void packageClosing ( PackageEvent ev ) 
    {}  
    
    /**
     * Returns if this extension is compatible with the current BlueJ-version
     */
    public boolean isCompatible () 
    { 
        return true; 
    }

    /**
     * Returns the version of this extension
     */
    public String  getVersion () 
    { 
        return ("08.2007");  
    }

    
    /**
     * Returns the name of this extension
     */
    public String  getName () 
    { 
        return ("SyntaxSourcePrinter");  
    }

    /** Is called when the extension terminates */
    public void terminate() {}
    
    /** Returns the description of the extension"
     */
    public String getDescription () 
    {
        return ("Prints the source text with syntax-highlighting.\n2007 by Marc Seidemann.");          
    }

    /**
     * Returns the URL for the support of the extension
     */
    public URL getURL () 
    {
        try 
        {
            return new URL("");
        } 
        catch ( Exception eee )
        {
            return null;
        }
    }
}

/**
 * Creates the panel for the preferences of this extension
 */
class Preferences implements PreferenceGenerator {

    private static FontDialog fd;
    private BlueJ bluej;
    public static final String FONT_SIZE="PrintFontSize";
    public static final String FONT_NAME="PrintFontName";
    public static final String FONT_STYLE="PrintFontStyle";
    public static Font lastOpened;
    public static boolean printLineNumbers;
    
    /** The Constructor */
    public Preferences(BlueJ bluej) {
        this.bluej = bluej;
        
        fd = new FontDialog();
        // Loads the saved values
        loadValues();
    }
    /** Returns the panel */
    public JPanel getPanel ()  { return fd; }

    /** Saves the values to the bluej.properties */
    public void saveValues () 
    {
        bluej.setExtensionPropertyString(FONT_SIZE, fd.getFontsize());
        bluej.setExtensionPropertyString(FONT_NAME, fd.getFontname());
        bluej.setExtensionPropertyString(FONT_STYLE, fd.getStyle());
        lastOpened = fd.getPrintFont();
        printLineNumbers = fd.getLinenumberPrinting();
    }

    /** Loads the values from the bluej.properties */
    public void loadValues () 
    {

        fd.setFontsize(bluej.getExtensionPropertyString(FONT_SIZE,"8"));
        fd.setFontname(bluej.getExtensionPropertyString(FONT_NAME,"SansSerif"));
        fd.setStyle(bluej.getExtensionPropertyString(FONT_STYLE,""));
        lastOpened = fd.getPrintFont();
        printLineNumbers = fd.getLinenumberPrinting();
    }
}//End of Class Preferences


/**
* Creates the MenuItems for the class-menu
*/
class MenuBuilder extends MenuGenerator {
    private BPackage curPackage;
    private BClass curClass;
    private BObject curObject;
    private String fname;
    private BlueJ bluej;
    
    public MenuBuilder(BlueJ bluej)
    {
        this.bluej = bluej;
    }
    
    /** Returns the menuitem for the print-action */
    public JMenuItem getClassMenuItem(BClass aClass)
    {
        JMenu menu = new JMenu("SyntaxSourcePrinter");
        JMenuItem item = new JMenuItem(new SimpleAction("Print", "Class: menu"));
        item.setActionCommand("pr");
        menu.add(item);
        item = new JMenuItem(new SimpleAction("Export to HTML", "Class: menu"));
        item.setActionCommand("ex");
        menu.add(item);
        item = new JMenuItem(new SimpleAction("Export to RTF", "Class: menu"));
        item.setActionCommand("rtf");
        menu.add(item);
        return menu;
    }

    /** The classmenu has been selected and the name of the class is detected */
    public void notifyPostClassMenu(BClass bc, JMenuItem jmi) 
    {
        curPackage = null ; curClass = bc ; curObject = null;
    }

    /** The "Print"-Item has been selected */
    private void itemClicked(String cmd) {
        try {

            if (curClass != null)
                curPackage = curClass.getPackage();

            fname = curPackage.getDir().getAbsolutePath()+System.getProperty("file.separator")+curClass.getName()+".java";
            if (cmd.equals("pr"))
                print(fname);
             else
             if (cmd.equals("ex"))
                export(fname);
             else
             if (cmd.equals("rtf"))
                exportRTF(fname);
        } catch (Exception exc) { }
    }

    /** The Package will be reloaded */
    public void reload()
    {
        try
        {
            curPackage.reload();
        }
        catch(Exception e)
        {}
    }
    
    /** Prints the source text of the choosen class */
    private void print(String fname) 
    {
        new DocumentCreator(fname,Preferences.lastOpened,curClass.getName(),Preferences.printLineNumbers,bluej);
    }
    
    /** Exports the source text of the choosen class to a HTML file */
    private void export(String fname)
    {
        Export.initHTML(fname,Preferences.lastOpened,curClass.getName(),Preferences.printLineNumbers);
    }
    
    /** Exports the source text of the choosen class to a HTML file */
    private void exportRTF(String fname)
    {
        Export.initRTF(fname,Preferences.lastOpened,curClass.getName());
    }
    
    
    /** The Menu-Actions */
    class SimpleAction extends AbstractAction 
    {
        
        public SimpleAction(String menuName, String msg) 
        {
            putValue(AbstractAction.NAME, menuName);

        }
        public void actionPerformed(ActionEvent e) 
        {
                itemClicked(e.getActionCommand());
        }
    }//End of Class SimpleAction

}//End pf Class MenuBuilder